import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class CustomInputText extends StatefulWidget {
  final String placeholder;
  final IconData icon;
  final TextEditingController textController;
  final TextInputType keyboardType;
  final bool isPassword;
  final bool isAName;
  final bool isAPhone;
  final Function onChanged;
  final bool validation;
  final bool enabled;
  final Color backgroundColor;
  final Color onfocusColor;
  final bool shadow;
  final double borderRadius;

  const CustomInputText({
    @required this.placeholder,
    @required this.textController,
    this.icon,
    this.keyboardType = TextInputType.text,
    this.isPassword = false,
    this.isAName = false,
    this.isAPhone = false,
    this.onChanged,
    this.validation,
    this.enabled = true,
    this.backgroundColor = Colors.white,
    this.onfocusColor = Colors.blue,
    this.shadow = true,
    this.borderRadius = 30.0,
  });

  @override
  _CustomInputTextState createState() => _CustomInputTextState();
}

class _CustomInputTextState extends State<CustomInputText> {
  bool dontShow = true;
  bool emailValid = true;

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: (widget.isPassword)
          ? EdgeInsets.only(right: 0)
          : (widget.icon == null)
              ? EdgeInsets.only(left: 15, right: 15)
              : EdgeInsets.only(right: 15),
      decoration: BoxDecoration(
          color: widget.backgroundColor,
          border: (widget.keyboardType == TextInputType.emailAddress &&
                  widget.onChanged == null)
              ? Border.all(
                  color: (emailValid) ? Colors.black26 : Colors.red[600],
                )
              : (widget.validation != null)
                  ? Border.all(
                      color: (widget.validation) ? Colors.green : Colors.red,
                    )
                  : Border.all(color: Colors.black26),
          borderRadius: BorderRadius.circular(widget.borderRadius),
          boxShadow: <BoxShadow>[
            if (widget.shadow)
              BoxShadow(
                  color: Colors.black.withOpacity(0.05),
                  offset: Offset(0, 5),
                  blurRadius: 5)
          ]),
      child: Theme(
        data: Theme.of(context).copyWith(
          // override textfield's icon color when selected
          primaryColor: widget.onfocusColor,
        ),
        child: (widget.isPassword)
            ? Container(
                padding: EdgeInsets.only(right: 10),
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  children: [
                    Expanded(
                      child: Container(
                        child: TextField(
                          controller: widget.textController,
                          onChanged: widget.onChanged,
                          cursorColor: Colors.yellow[900],
                          autocorrect: false,
                          keyboardType: widget.keyboardType,
                          obscureText: dontShow,
                          decoration: InputDecoration(
                              prefixIcon: Icon(widget.icon),
                              focusedBorder: InputBorder.none,
                              border: InputBorder.none,
                              hintText: widget.placeholder),
                        ),
                      ),
                    ),
                    IconButton(
                        icon: Icon(
                          (dontShow)
                              ? Icons.visibility_off
                              : Icons.visibility_rounded,
                          color: Colors.black45,
                        ),
                        onPressed: () {
                          setState(() {
                            dontShow = !dontShow;
                          });
                        })
                  ],
                ),
              )
            : (widget.icon != null)
                ? TextField(
                    enabled: widget.enabled,
                    controller: widget.textController,
                    onChanged:
                        (widget.keyboardType == TextInputType.emailAddress &&
                                widget.onChanged == null)
                            ? (value) {
                                final isValid = validateEmail(value);
                                setState(() {
                                  emailValid = isValid;
                                });
                              }
                            : widget.onChanged,
                    cursorColor: widget.onfocusColor,
                    textCapitalization:
                        (widget.keyboardType == TextInputType.emailAddress)
                            ? TextCapitalization.none
                            : (widget.isAName)
                                ? TextCapitalization.words
                                : TextCapitalization.none,
                    autocorrect: false,
                    keyboardType: widget.keyboardType,
                    inputFormatters: [
                      if (widget.isAPhone)
                        FilteringTextInputFormatter.allow(RegExp(r'[0-9]')),
                      if (widget.isAName)
                        FilteringTextInputFormatter.allow(
                            RegExp(r'[a-z\A-Z\á\é\í\ó\ú\Á\É\Í\Ó\Ú\ ]'))
                    ],
                    decoration: InputDecoration(
                        prefixIcon: Icon(widget.icon),
                        focusedBorder: InputBorder.none,
                        border: InputBorder.none,
                        hintText: widget.placeholder),
                  )
                : TextField(
                    enabled: widget.enabled,
                    controller: widget.textController,
                    onChanged:
                        (widget.keyboardType == TextInputType.emailAddress &&
                                widget.onChanged == null)
                            ? (value) {
                                final isValid = validateEmail(value);
                                setState(() {
                                  emailValid = isValid;
                                });
                              }
                            : widget.onChanged,
                    cursorColor: widget.onfocusColor,
                    textCapitalization:
                        (widget.keyboardType == TextInputType.emailAddress)
                            ? TextCapitalization.none
                            : (widget.isAName)
                                ? TextCapitalization.words
                                : TextCapitalization.none,
                    autocorrect: false,
                    keyboardType: widget.keyboardType,
                    inputFormatters: [
                      if (widget.isAPhone)
                        FilteringTextInputFormatter.allow(RegExp(r'[0-9]')),
                      if (widget.isAName)
                        FilteringTextInputFormatter.allow(
                            RegExp(r'[a-z\A-Z\á\é\í\ó\ú\Á\É\Í\Ó\Ú\ ]'))
                    ],
                    decoration: InputDecoration(
                        focusedBorder: InputBorder.none,
                        border: InputBorder.none,
                        hintText: widget.placeholder),
                  ),
      ),
    );
  }
}

bool validateEmail(String email) {
  Pattern pattern =
      r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$';
  RegExp regExp = new RegExp(pattern);
  if (regExp.hasMatch(email)) {
    return true;
  } else {
    return false;
  }
}
